#include "microcontroller.h"

#include <chrono>
#include <thread>

static constexpr bool debug = true;

void Microcontroller::relayToggle(int state, int relay)
{
    char buffer[8];
    int length = sprintf(buffer, "%d \n", relay);
    state ? write("relay on ") : write("relay off ");
    write(buffer, length);
}

void Microcontroller::relayOn(int relay)
{
  relayToggle(true, relay);
}

void Microcontroller::relayOff(int relay)
{
  relayToggle(false, relay);
}

//rgb lights

void Microcontroller::rgbOn()
{
    write("rgb on\n");
}

void Microcontroller::rgbOff()
{
    write( "rgb off\n");
}

void Microcontroller::changeRgbColor(const QColor color)
{
    char buffer[64];
    int length = sprintf(buffer, "rgb set %03d %03d %03d\n", color.red(), color.green(), color.blue());
        write(buffer, length);
        std::cout<<buffer;
    }

    void Microcontroller::setAuxPwm(int duty)
    {
    char buffer[64];
    int length = sprintf(buffer, "aux set %03d\n", duty );
    write(buffer, length);
}

void Microcontroller::write(const QByteArray& buffer)
{
    #ifndef Q_OS_ANDROID
    if constexpr(debug) std::cerr<<buffer.data();
    #endif
    if(_port != nullptr)
    {
        _port->write(buffer);
        _port->waitForBytesWritten(1000);
    }
    std::this_thread::sleep_for(std::chrono::milliseconds(40));
}

void Microcontroller::write(char* buffer, const size_t length)
{
    #ifndef Q_OS_ANDROID
    if constexpr(debug) std::cerr<<buffer;
    #endif
    if(_port != nullptr)
    {
        _port->write(buffer, length);
        _port->waitForBytesWritten(1000);
    }
    std::this_thread::sleep_for(std::chrono::milliseconds(40));
}

void Microcontroller::setPattern(int pattern)
{
        char buffer[4];
        write("rgb pattern ");
        int length = sprintf(buffer, "%d\n", pattern);
        write(buffer, length);
}

void Microcontroller::startSunrise()
{
    setPattern(4);
}

bool Microcontroller::connected()
{
   if(_port != nullptr) return _port->isOpen();
   else return false;
}

void Microcontroller::run()
{
    abort();

    loop.reset(new QEventLoop);
    QTimer timer;
    connect(&timer, SIGNAL(timeout()), this, SLOT(doTick()));
    timer.setInterval(500);
    timer.start();

    loop->exec();
}

void Microcontroller::requestState()
{
   write("state\n");
}

void Microcontroller::abort()
{
    if (!loop.isNull())
    {
        loop->quit();
    }
}

//housekeeping

Microcontroller::Microcontroller(QIODevice* port): _port(port)
{
    _port->readAll();
    _port->write("\n");
}

Microcontroller::Microcontroller()
{
}

Microcontroller::~Microcontroller()
{
    if(_port != nullptr) delete _port;
}

void Microcontroller::setIODevice(QIODevice *port)
{
    _port = port;
    _port->readAll();
    _port->write("\n");
}

std::shared_ptr<Relay> Microcontroller::processRelayLine(const QString& buffer)
{
    QStringList bufferList = buffer.split(' ');
    if(bufferList.size() >= 8 && buffer.startsWith("RELAY NUMBER:"))
    {
        QString name;
        for(int i = 8; i < bufferList.size(); i++) name.append(bufferList[i] + ' ');
        if(name.size() > 1)name.remove(name.size()-1, 1);
        else name = "Relay " + QString::number(bufferList[1].toInt(nullptr, 2));
        qDebug()<<"Relay "<<bufferList[2].toInt()<<"Name "<<name<<" id "<<bufferList[4].toInt(nullptr, 2)<<" state "<<bufferList[6].toInt();
        return std::shared_ptr<Relay>( new Relay(nullptr, this, bufferList[2].toInt(), name, bufferList[4].toInt(nullptr, 2), bufferList[6].toInt()));
    }
    return  nullptr;
}

void Microcontroller::processList(const QString& buffer)
{
    QStringList bufferList = buffer.split(' ');
    if(bufferList.size() >= 8 && buffer.startsWith("RELAY NUMBER:"))
    {
        relayList.push_back(processRelayLine(buffer));
    }
    else if(buffer.contains("EOL"))
    {
        listMode = false;
        qDebug()<<"got relay list " << relayList.size();
        gotRelayList(relayList);
        relayList.clear();
    }
    else listMode = false;
}

void Microcontroller::processRelayState(const QString& buffer)
{
    itemChanged(static_cast<ItemData>(*processRelayLine(buffer)));
}

void Microcontroller::processSensorState(const QString& buffer)
{
    QStringList bufferList = buffer.split(' ');
    Sensor sensor(bufferList[2].toUInt(), bufferList[4].toUInt(), bufferList[6].toUInt());
    if(sensor.type == Sensor::TYPE_HUMIDITY ||  sensor.type == Sensor::TYPE_TEMPERATURE) sensor.field = sensor.field/10;
    if(bufferList.size() >= 7)gotSensorState(sensor);
}


void Microcontroller::processMicroReturn()
{
    qDebug()<<_buffer;
    if(listMode) processList(_buffer);
    else
    {
        if(_buffer.startsWith("Relays:"))
        {
            listMode = true;
            relayList.clear();
        }
        else if(_buffer.startsWith("RELAY NUMBER:"))processRelayState(_buffer);
        else if(_buffer.startsWith("SENSOR")) processSensorState(_buffer);
    }

}

void Microcontroller::doTick()
{
    if(_port != nullptr)
    {
        char charBuf;
        while(_port->getChar(&charBuf))
        {
            _buffer.push_back(charBuf);
            if( _buffer.endsWith('\n') )
            {
                _buffer.remove('\n');
                processMicroReturn();
                textRecived(_buffer);
                _buffer.clear();
            }
        }
    }
}
